@@ -1,11 +1,15 @@
+import re
 import gzip
 import json
 import traceback
+import numpy as np
 import SimpleITK as sitk
 
 from pathlib import Path
+from uuid import uuid4
 from loguru import logger
-from mircat_v2.configs import read_dbase_config
+from mircat_v2.configs import read_dbase_config, db_schema
+from mircat_v2.dicom_conversion import get_metadata
 
 
 class Nifti:
@@ -35,23 +39,56 @@ class Nifti:
             raise SimpleITKReadError()
         self.spacing = self.img.GetSpacing()
         self.shape = self.img.GetSize()
-        if (nifti_path.parent / "metadata.json").exists():
-            logger.debug(
-                f"{nifti_path.parent / 'metadata.json'} found. Loading metadata."
-            )
-            with (nifti_path.parent / "metadata.json").open() as f:
+        if (self.parent / "metadata.json").exists():
+            logger.trace(f"{self.parent / 'metadata.json'} found. Loading metadata.")
+            with (self.parent / "metadata.json").open() as f:
                 self.metadata: dict = json.load(f)
-        elif (nifti_path.parent / "header_info.json").exists():
-            logger.debug(
-                f"Old format metadata {nifti_path.parent / 'header_info.json'} found. Loading metadata."
+            if self.metadata.get("series_uid") is None:
+                self.metadata["series_uid"] = str(uuid4())
+                self._write_metadata()
+
+        elif (self.parent / "header_info.json").exists():
+            logger.trace(
+                f"Old format metadata {self.parent / 'header_info.json'} found. Loading metadata."
             )
-            with (nifti_path.parent / "header_info.json").open() as f:
+            with (self.parent / "header_info.json").open() as f:
                 self.metadata: dict = json.load(f)
+            logger.trace("Updating metadata in Nifti.__init__")
+            self.update_metadata_file()
         else:
             logger.debug(
-                f"No metadata.json file found next to {nifti_path}. Setting metadata to empty dictionary."
+                f"No metadata.json or header_info.json file found next to {nifti_path}. Setting metadata['series_uid'] to a file with a generated uuid."
             )
-            self.metadata: dict = {}
+            self.metadata: dict = {"series_uid": str(uuid4())}
+            self._write_metadata()
+
+    def update_metadata_file(self) -> None:
+        dicoms = list(self.parent.glob("*.dcm"))
+        if not dicoms:
+            logger.trace(
+                f"No .dcm extension file found in {self.parent} - looking for dicom without extension"
+            )
+            dicoms = [
+                f
+                for f in self.parent.iterdir()
+                if f.is_file() and re.match(r"^[\d.]+$", f.name)
+            ]
+            if not dicoms:
+                logger.trace(f"No reference dicom found in {self.parent}")
+                if self.metadata.get("series_number") is None:
+                    self.metadata["series_number"] = self.name.split("_")[0]
+                self.metadata["series_uid"] = str(uuid4())
+                # Delete the old version
+        if dicoms:
+            self.metadata = get_metadata(dicoms[0])
+            if self.metadata.get("series_uid") is None:
+                self.metadata["series_uid"] = str(uuid4())
+        # (self.parent / 'header_info.json').unlink()
+        self._write_metadata()
+
+    def _write_metadata(self):
+        with (self.parent / "metadata.json").open("w") as f:
+            json.dump(self.metadata, f, indent=2)
 
 
 class SegNifti(Nifti):
@@ -73,7 +110,7 @@ class SegNifti(Nifti):
             interpolator_type=interpolator_type,
         )
         sitk.WriteImage(resampled, output_path)
-        logger.debug("Successfully resampled and wrote to {}", output_path)
+        logger.debug("Successfully resampled {} and wrote to {}", self.path, output_path)
 
 
 class StatsNifti(Nifti):
@@ -84,22 +121,22 @@ class StatsNifti(Nifti):
         self.stats_file = stats_file
 
         if stats_file.exists() and not overwrite:
-            logger.debug(f"Stats file {stats_file} found. Loading statistics.")
+            logger.trace(f"Stats file {stats_file} found. Loading statistics.")
             try:
                 with stats_file.open() as f:
                     self.stats = json.load(f)
             except json.JSONDecodeError:
-                logger.error(
+                logger.warning(
                     f"Error decoding JSON from {stats_file}. Initializing empty stats."
                 )
                 self.stats = {}
         else:
             if not overwrite:
-                logger.debug(
+                logger.trace(
                     f"No stats file found at {stats_file}. Setting stats to empty dictionary."
                 )
             else:
-                logger.debug(
+                logger.trace(
                     f"Stats file {stats_file} exists but overwrite is set to True. Initializing empty stats."
                 )
             self.stats = {}
@@ -107,9 +144,13 @@ class StatsNifti(Nifti):
         # We want this initialized regardless of it's it's empty or not
         self.add_stats("metadata", self.metadata)
         if self.metadata != {}:
-            logger.debug(f"Merging metadata with stats for {self.name}.")
-            self.add_stats("series_uid", self.metadata.get("series_uid", None))
-            self.add_stats("study_uid", self.metadata.get("study_uid", None))
+            logger.trace(f"Merging metadata with stats for {self.name}.")
+            study_uid = self.metadata.get("study_uid")
+            series_uid = self.metadata.get("series_uid")
+            if series_uid is None:
+                series_uid = self._generate_series_uid()
+            self.add_stats("series_uid", series_uid)
+            self.add_stats("study_uid", study_uid)
         # TODO - currently vertebrae midlines will load regardless of the desired stats resolution - which is not correct.
         # If the stats resolution is different from the nifti resolution, we must remeasure the midlines.
         if self.stats.get("vertebrae") is not None:
@@ -146,7 +187,7 @@ class StatsNifti(Nifti):
     def add_stats(self, key: str, value: any, subdict=None) -> None:
         """Add a key-value pair to the stats dictionary."""
         if key in self.stats:
-            logger.debug(
+            logger.trace(
                 f"Key {key} already exists in stats for {self.path}. Overwriting value."
             )
         if subdict:
@@ -158,10 +199,12 @@ class StatsNifti(Nifti):
     def preprocess_for_stats(
         self, new_spacing, image_resampler, label_resampler
     ) -> None:
-        logger.debug(
-            "Resampling image and segmentations for nifti {} to new spacing {}",
+        logger.trace(
+            "Resampling image and segmentations for nifti {} to new spacing {} using {} for images and {} for labels",
             self.name,
             new_spacing,
+            image_resampler,
+            label_resampler,
         )
         self.img = resample_with_sitk(
             self.img,
@@ -170,14 +213,12 @@ class StatsNifti(Nifti):
             interpolator_type=image_resampler,
             reorient="LAI",
         )
+        logger.trace("image resampled")
         self.spacing = self.img.GetSpacing()
         self.shape = self.img.GetSize()
         self.add_stats("stats_resolution", new_spacing)
         new_id_to_seg_map = {}
         for seg_id, seg_path in self.id_to_seg_map.items():
-            logger.debug(
-                "Resampling segmentation for seg_id {} at path {}", seg_id, seg_path
-            )
             label = sitk.ReadImage(seg_path)
             resampled_label = resample_with_sitk(
                 label,
@@ -216,34 +257,49 @@ class StatsNifti(Nifti):
 
     def save_json_stats(self, gzip_file: bool = False) -> None:
         """Save the statistics to a JSON file."""
-        save_stats = {k: v for k, v in self.stats.items() if "-db" not in k}
         if gzip_file:
-            logger.debug(f"Compressing stats file {self.stats_file} with gzip.")
+            logger.trace(f"Compressing stats file {self.stats_file} with gzip.")
             self.add_stats(
                 "output_stats_file",
                 str(self.stats_file.with_suffix(".json.gz")),
                 "metadata",
             )
+            save_stats = {k: self._make_json_serializable(v) for k, v in self.stats.items()}
             with gzip.open(
                 self.stats_file.with_suffix(".json.gz"), "wt", encoding="utf-8"
             ) as f:
                 json.dump(save_stats, f, indent=2)
         else:
+            logger.trace(f"Writing stats file {self.stats_file} without compression")
             self.add_stats("output_stats_file", str(self.stats_file), "metadata")
+            save_stats = {k: self._make_json_serializable(v) for k, v in self.stats.items()}
             with self.stats_file.open("w") as f:
                 json.dump(save_stats, f, indent=2)
 
+    def _make_json_serializable(self, obj):
+        """Convert non-JSON serializable objects to serializable forms."""
+        if isinstance(obj, np.ndarray):
+                return obj.tolist()
+        elif isinstance(obj, np.integer):
+            return int(obj)
+        elif isinstance(obj, np.floating):
+            return float(obj)
+        elif isinstance(obj, (np.bool_, bool)):
+            return bool(obj)
+        elif isinstance(obj, dict):
+            return {k: self._make_json_serializable(v) for k, v in obj.items()}
+        elif isinstance(obj, (list, tuple)):
+            return [self._make_json_serializable(item) for item in obj]
+        elif hasattr(obj, '__dict__'):
+            # For custom objects, try to convert to dict
+            return self._make_json_serializable(obj.__dict__)
+        else:
+            return obj  
+
     def format_stats_for_db(self):
         """Format the stats for database insertion. The output dictionary will have keys matching the table names. This function also has internal functions defined in its scope"""
         stats_tables = [
-            "metadata",
-            "vol_int",
-            "contrast",
-            "vertebrae",
-            "aorta_metrics",
-            "aorta_diameters",
-            "tissues_volumetric",
-            "tissues_vertebral",
+            key for key in db_schema if key not in ["conversions", "segmentations"]
         ]
         formatted_stats: dict[str, list[tuple]] = {}
         nifti_path = str(self.path)
@@ -256,7 +312,7 @@ class StatsNifti(Nifti):
             """Format the metadata for database insertion."""
             metadata = self.stats.get("metadata", {})
             if not metadata:
-                logger.warning(f"No metadata found in the stats for {self.path}")
+                logger.debug(f"No metadata found in the stats for {self.path}")
                 return []
             columns = list(tables["metadata"].keys())
             row_data = {}
@@ -272,12 +328,14 @@ class StatsNifti(Nifti):
                 if value is None:
                     if col == "birth_date":
                         value = metadata.get("birthday")
-                elif col == "pixel_length_mm":
-                    value = metadata.get("length_mm")
-                elif col == "pixel_width_mm":
-                    value = metadata.get("width_mm")
-                elif col == "age":
-                    value = metadata.get("age_at_scan")
+                    elif col == "pixel_length_mm":
+                        value = metadata.get("length_mm")
+                    elif col == "pixel_width_mm":
+                        value = metadata.get("width_mm")
+                    elif col == "age":
+                        value = metadata.get("age_at_scan")
+                    elif col == "model":
+                        value = metadata.get("manufacturer_model")
                 row_data[col] = value
             return [row_data]
 
@@ -285,7 +343,7 @@ class StatsNifti(Nifti):
             "Format the vol_int data for database insertion"
             vol_int = self.stats.get("vol_int", {})
             if not vol_int:
-                logger.warning(f"No vol_int stats found in {self.path}")
+                logger.debug(f"No vol_int stats found in {self.path}")
                 return []
             rows = []
             for structure, stats in vol_int.items():
@@ -303,7 +361,7 @@ class StatsNifti(Nifti):
         def _format_contrast() -> list[dict]:
             contrast = self.stats.get("contrast", {})
             if not contrast:
-                logger.warning(f"No contrast stats found in {self.path}")
+                logger.debug(f"No contrast stats found in {self.path}")
                 return []
             return [
                 {
@@ -318,7 +376,7 @@ class StatsNifti(Nifti):
         def _format_vertebrae() -> list[dict]:
             vertebrae = self.stats.get("vertebrae", {})
             if not vertebrae:
-                logger.warning(f"No vertebrae stats found in {self.path}")
+                logger.debug(f"No vertebrae stats found in {self.path}")
                 return []
             rows = []
             for vertebra, stats in vertebrae.items():
@@ -341,7 +399,7 @@ class StatsNifti(Nifti):
             else:
                 kvp = 0
             if not aorta_metrics:
-                logger.warning(f"No aorta metrics found in {self.path}")
+                logger.debug(f"No aorta metrics found in {self.path}")
                 return []
             rows = []
             for region, stats in aorta_metrics.items():
@@ -373,7 +431,7 @@ class StatsNifti(Nifti):
         def _format_aorta_diameters() -> list[dict]:
             aorta_metrics = self.stats.get("aorta", {})
             if not aorta_metrics:
-                logger.warning(f"No aorta diameters found in {self.path}")
+                logger.debug(f"No aorta diameters found in {self.path}")
                 return []
             rows = []
             measure_keys = ["max", "min", "mid", "proximal", "distal"]
@@ -405,7 +463,7 @@ class StatsNifti(Nifti):
         def _format_tissues_volumetric() -> list[dict]:
             volumetric = self.stats.get("tissues", {}).get("volumetric", {})
             if not volumetric:
-                logger.warning(f"No tissue volumetric stats found in {self.path}")
+                logger.debug(f"No tissue volumetric stats found in {self.path}")
                 return []
             rows = []
             for region, stats in volumetric.items():
@@ -425,7 +483,7 @@ class StatsNifti(Nifti):
         def _format_tissues_vertebral() -> list[dict]:
             vertebral = self.stats.get("tissues", {}).get("vertebral", {})
             if not vertebral:
-                logger.warning(f"No tissue vertebral stats found in {self.path}")
+                logger.debug(f"No tissue vertebral stats found in {self.path}")
                 return []
             rows = []
             for vertebra, stats in vertebral.items():
@@ -442,6 +500,28 @@ class StatsNifti(Nifti):
                         )
             return rows
 
+        def _format_iliac() -> list[dict]:
+            rows = []
+            iliac = self.stats.get("iliac", {})
+            if not iliac:
+                logger.debug(f"No iliac stats found in {self.path}")
+                return rows
+            for side, data in iliac.items():
+                length_mm = data.pop("length_mm", 0.0)
+                for location, metrics in data.items():
+                    for metric, value in metrics.items():
+                        rows.append(
+                            {
+                                **identifier,
+                                "side": side,
+                                "length_mm": length_mm,
+                                "location": location,
+                                "metric": metric,
+                                "value": value,
+                            }
+                        )
+            return rows
+
         for table in stats_tables:
             match table:
                 case "metadata":
@@ -460,6 +540,8 @@ class StatsNifti(Nifti):
                     table_data = _format_tissues_volumetric()
                 case "tissues_vertebral":
                     table_data = _format_tissues_vertebral()
+                case "iliac":
+                    table_data = _format_iliac()
             formatted_stats[table] = table_data
         self.db_stats = formatted_stats
         return self
@@ -553,7 +635,7 @@ def resample_with_sitk(
             )
         )
         if current_orientation != reorient:
-            logger.debug(f"Reorienting image from {current_orientation} to {reorient}.")
+            logger.trace(f"Reorienting image from {current_orientation} to {reorient}.")
             reorienter = sitk.DICOMOrientImageFilter()
             reorienter.SetDesiredCoordinateOrientation(reorient)
             resampled_image = reorienter.Execute(resampled_image)
