@@ -18,9 +18,9 @@ def calculate_aorta_stats(nifti, seg_id: str, label_map: dict) -> dict:
     :param label_map: Dictionary mapping structure names to label IDs.
     :return: Dictionary containing aorta specific statistics.
     """
-    logger.debug(f"Calculating aorta stats for {nifti.name}.")
+    logger.trace(f"Calculating aorta stats for {nifti.name}.")
     if not hasattr(nifti, "vertebrae_midlines"):
-        logger.warning(
+        logger.debug(
             f"Vertebrae midlines not found in {nifti.name}. Skipping aorta stats calculation."
         )
         return {}
@@ -390,7 +390,7 @@ class Aorta(Vessel):
             )
             # We use the anatomical method if the 2 extra segmentations were found
             if np.all(np.isin([brach_label, subclavian_label], self.cpr)):
-                logger.debug("Using aortic CPR to define aortic arch")
+                logger.trace("Using aortic CPR to define aortic arch")
                 # We shrink this so we don't include the abdominal aorta
                 thoracic_cpr = self.cpr[thoracic_indices]
                 # The brachiocephalic trunk signifies the start of the aortic arch
@@ -404,7 +404,7 @@ class Aorta(Vessel):
                         arch_end = len(thoracic_cpr) - idx
             # Otherwise, we use the morphology of the aorta to define the regions
             else:
-                logger.debug("Using aortic morphology to define aortic arch")
+                logger.trace("Using aortic morphology to define aortic arch")
                 # Need at least 50mm2 to be considered
                 min_segmentation_area = 50
                 # This is the peak of the centerline
@@ -426,7 +426,6 @@ class Aorta(Vessel):
                             split = idx
                             break
                 if split is None:
-                    logger.error("Aortic arch could not be defined from morphology")
                     raise ValueError("Aortic arch could not be defined from morphology")
                 # We shrink this to exclude the abdominal aorta
                 thoracic_centerline = self.centerline.coordinates[thoracic_indices]
@@ -439,7 +438,6 @@ class Aorta(Vessel):
                         arch_end = len(thoracic_centerline) - idx
                         break
                 if arch_start is None or arch_end is None:
-                    logger.error("Aortic arch could not be defined from morphology")
                     raise ValueError("Aortic arch could not be defined from morphology")
             # extract the aortic root as a separate region from the ascending aorta
             ascending_start = 0
@@ -484,11 +482,11 @@ class Aorta(Vessel):
             for index in indices:
                 self.cross_section_stats[index].anatomical_region = region
             if len(indices) < 3:
-                logger.warning(
+                logger.debug(
                     f"Skipping aortic region {region} with less than 3 indices."
                 )
                 continue
-            logger.debug(f"Aggregating stats for aortic region: {region}")
+            logger.trace(f"Aggregating stats for aortic region: {region}")
             # we only run these if the whole thing is there
             if region in ["root", "asc", "arch"]:
                 entire_region = 1
@@ -582,11 +580,11 @@ class Aorta(Vessel):
         self, region: list[AortaStraightenedCPRCrossSection]
     ) -> dict:
         diameters = {}
-        # Filter out any cross section missing diameters or too small to be sure about
+        # Filter out any cross section missing diameters or too small/uncircular to be sure about
         diam_cross_sections = []
         for cs in region:
             if cs.mean_diameter is not None:
-                if cs.mean_diameter > 5:
+                if cs.mean_diameter > 5 and cs.roundness >= 0.7:
                     diam_cross_sections.append(cs)
         # if it's empty, return the empty dictionary
         if len(diam_cross_sections) == 0:
