@@ -1,3 +1,4 @@
+from dataclasses import dataclass
 import SimpleITK as sitk
 import numpy as np
 import xs3d
@@ -100,7 +101,7 @@ class Vessel:
         :return: The Vessel instance with the centerline created.
         """
         if self.skeleton is None:
-            logger.debug("Skeleton not created yet. Creating skeleton first.")
+            logger.trace("Skeleton not created yet. Creating skeleton first.")
             self.create_skeleton(**kwargs)
             if self.skeleton is None:
                 raise ValueError(
@@ -432,3 +433,15 @@ def _closest_to_centroid(cross_section: np.ndarray, regions: list) -> np.ndarray
     center_label = np.zeros_like(cross_section, dtype=np.uint8)
     center_label[center_region.coords[:, 0], center_region.coords[:, 1]] = 1
     return center_label
+
+
+@dataclass
+class CrossSectionStat:
+    index: int
+    mean_diameter: float | None
+    major_diameter: float | None
+    minor_diameter: float | None
+    area: float | None
+    flatness: float | None
+    roundness: float | None
+    centroid: tuple | None
